import Vuex from 'vuex'
import Vue from 'vue'
import ax from './api'

import gradySays from './grady_speak'

Vue.use(Vuex)

const store = new Vuex.Store({
  state: {
    token: '',
    loggedIn: false,
    username: '',
    error: '',
    examInstance: 'B.Inf 1301 Kohorte 2'
  },
  getters: {
    gradySpeak: state => {
      return gradySays[Math.floor(Math.random() * gradySays.length)]
    }
  },
  mutations: {
    'API_FAIL': function (state, error) {
      state.error = error
    },
    'SET_JWT_TOKEN': function (state, token) {
      state.token = token
      ax.defaults.headers.common['Authorization'] = 'JWT ' + token
    },
    'LOGIN': function (state, username) {
      state.loggedIn = true
      state.username = username
    },
    'LOGOUT': function (state) {
      state.loggedIn = false
    }
  },
  actions: {
    async getJWTToken (context, credentials) {
      try {
        const response = await ax.post('api-token-auth/', credentials)
        context.commit('LOGIN', credentials.username)
        context.commit('SET_JWT_TOKEN', response.data.token)
      } catch (error) {
        if (error.response) {
          const errorMsg = 'Unable to log in with provided credentials.'
          context.commit('API_FAIL', errorMsg)
          throw errorMsg
        } else {
          const errorMsg = 'Cannot reach server.'
          context.commit('API_FAIL', errorMsg)
          throw errorMsg
        }
      }
    },
    logout (store) {
      store.commit('LOGOUT')
      store.commit('SET_JWT_TOKEN', '')
    }
  }
})

export default store
